Package: Boruta
Title: Wrapper Algorithm for All Relevant Feature Selection
Version: 10.0.0
Imports: fru
Suggests: rFerns, randomForest, ranger, survival
Authors@R: c(
  person(
   c("Miron","Bartosz"),"Kursa",
   role=c("aut","cre"),
   email="m@mbq.me",
   comment=c(ORCID="0000-0001-7672-648X")
  ),
  person(
   c("Witold","Remigiusz"),"Rudnicki",
   role="aut"
  )
 )
Description: An all relevant feature selection wrapper algorithm.
 It finds relevant features by comparing original attributes' importance with importance achievable at random, estimated using their permuted copies (shadows).
License: GPL (>= 2)
URL: https://gitlab.com/mbq/Boruta/
RoxygenNote: 7.3.3
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2026-05-10 22:18:46 UTC; mbq
Author: Miron Bartosz Kursa [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-7672-648X>),
  Witold Remigiusz Rudnicki [aut]
Maintainer: Miron Bartosz Kursa <m@mbq.me>
Repository: CRAN
Date/Publication: 2026-05-16 08:10:16 UTC
Built: R 4.5.2; ; 2026-05-16 12:20:49 UTC; unix
