# Generated by roxygen2: do not edit by hand

S3method(Boruta,default)
S3method(Boruta,formula)
S3method(plot,Boruta)
S3method(print,Boruta)
export(Boruta)
export(TentativeRoughFix)
export(attStats)
export(conditionalTransdapter)
export(decohereTransdapter)
export(getConfirmedFormula)
export(getImpFerns)
export(getImpFruRaw)
export(getImpFruZ)
export(getImpLegacyRfGini)
export(getImpLegacyRfRaw)
export(getImpLegacyRfZ)
export(getImpRfGini)
export(getImpRfRaw)
export(getImpRfZ)
export(getNonRejectedFormula)
export(getSelectedAttributes)
export(imputeTransdapter)
export(plotImpHistory)
